/*
 * Copyright 2019 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.android.material.datepicker;

import com.google.android.material.test.R;

import static org.hamcrest.core.Is.is;
import static org.hamcrest.core.IsNull.nullValue;
import static org.junit.Assert.assertThat;

import android.content.Context;
import androidx.appcompat.app.AppCompatActivity;
import android.widget.GridView;
import androidx.test.core.app.ApplicationProvider;
import com.google.android.material.internal.ParcelableTestUtils;
import java.util.Calendar;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.robolectric.Robolectric;
import org.robolectric.RobolectricTestRunner;
import org.robolectric.annotation.internal.DoNotInstrument;

@RunWith(RobolectricTestRunner.class)
@DoNotInstrument
public class SingleDateSelectorTest {

  private SingleDateSelector singleDateSelector;
  private MonthAdapter adapter;

  @Before
  public void setupMonthAdapters() {
    ApplicationProvider.getApplicationContext().setTheme(R.style.Theme_MaterialComponents_Light);
    AppCompatActivity activity = Robolectric.buildActivity(AppCompatActivity.class).setup().get();
    Context context = activity.getApplicationContext();
    GridView gridView = new GridView(context);
    singleDateSelector = new SingleDateSelector();
    adapter =
        new MonthAdapter(
            Month.create(2016, Calendar.FEBRUARY),
            singleDateSelector,
            new CalendarConstraints.Builder().build(),
            /* dayViewDecorator= */ null);
    gridView.setAdapter(adapter);
  }

  @Test
  public void dateSelectorMaintainsSelectionAfterParceling() {
    int position = 8;
    assertThat(adapter.withinMonth(position), is(true));
    singleDateSelector.select(adapter.getItem(position));
    long expected = adapter.getItem(position);
    SingleDateSelector singleDateSelectorFromParcel =
        ParcelableTestUtils.parcelAndCreate(singleDateSelector, SingleDateSelector.CREATOR);
    assertThat(singleDateSelectorFromParcel.getSelection(), is(expected));
  }

  @Test
  public void nullDateSelectionFromParcel() {
    SingleDateSelector singleDateSelector =
        ParcelableTestUtils.parcelAndCreate(this.singleDateSelector, SingleDateSelector.CREATOR);
    assertThat(singleDateSelector.getSelection(), nullValue());
  }

  @Test
  public void setSelectionDirectly() {
    Calendar setTo = UtcDates.getUtcCalendar();
    setTo.set(2004, Calendar.MARCH, 5);
    singleDateSelector.setSelection(setTo.getTimeInMillis());
    Calendar resultCalendar = UtcDates.getUtcCalendar();

    resultCalendar.setTimeInMillis(singleDateSelector.getSelection());

    assertThat(resultCalendar.get(Calendar.DAY_OF_MONTH), is(5));
    assertThat(
        singleDateSelector
            .getSelectedDays()
            .contains(UtcDates.canonicalYearMonthDay(setTo.getTimeInMillis())),
        is(true));
    assertThat(
        singleDateSelector
            .getSelectedDays()
            .contains(
                UtcDates.canonicalYearMonthDay(UtcDates.getTodayCalendar().getTimeInMillis())),
        is(false));
  }
}
